"""Tests for MCP protocol implementation."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

from mcp_server_llmling import constants


if TYPE_CHECKING:
    from mcp_server_llmling.mcp_inproc_session import MCPInProcSession


@pytest.mark.asyncio
async def test_mcp_resource_operations(configured_client: MCPInProcSession) -> None:
    """Test MCP resource operations."""
    # List resources
    response = await configured_client.send_request("resources/list")
    assert "resources" in response
    resource_list = response["resources"]
    assert len(resource_list) >= 1
    test_resource = next(r for r in resource_list if r["description"] == "Test resource")

    # Read resource content
    response = await configured_client.send_request(
        "resources/read",
        {"uri": test_resource["uri"]},
    )
    assert "contents" in response
    assert len(response["contents"]) == 1
    content = response["contents"][0]
    assert content["text"] == "Test content"
    assert content["mimeType"] == "text/plain"


@pytest.mark.asyncio
async def test_mcp_tool_operations(configured_client: MCPInProcSession) -> None:
    """Test MCP tool operations."""
    # List tools
    tools = await configured_client.list_tools()
    assert len(tools) >= 1
    _example_tool = next(t for t in tools if t["name"] == "example")

    # Call tool
    args = {"text": "test", "repeat": 2}
    result = await configured_client.call_tool("example", args)
    assert result["content"][0]["text"] == "testtest"


@pytest.mark.asyncio
async def test_mcp_prompt_operations(configured_client: MCPInProcSession) -> None:
    """Test MCP prompt operations."""
    # List prompts
    prompts = await configured_client.list_prompts()
    assert len(prompts) >= 1
    test_prompt = next(p for p in prompts if p["name"] == "test")

    # Get prompt
    params = {"name": test_prompt["name"], "arguments": {"test": "value"}}
    result = await configured_client.send_request("prompts/get", params)
    assert "messages" in result
    assert len(result["messages"]) >= 1
    assert result["messages"][0]["content"]["text"] == "test"


@pytest.mark.asyncio
async def test_mcp_error_handling(configured_client: MCPInProcSession) -> None:
    """Test MCP error response format."""
    # Test with invalid tool
    params = {"name": "nonexistent"}
    response = await configured_client.send_request("tools/call", params)
    assert "content" in response
    assert len(response["content"]) == 1
    assert "not found" in response["content"][0]["text"].lower()

    # Test with invalid prompt
    params = {"name": "nonexistent"}
    with pytest.raises(Exception) as exc_info:  # noqa: PT011
        await configured_client.send_request("prompts/get", params)
    assert "not found" in str(exc_info.value).lower()


@pytest.mark.asyncio
async def test_mcp_handshake(configured_client: MCPInProcSession) -> None:
    """Test MCP protocol handshake."""
    # Do another handshake to explicitly test it
    init_response = await configured_client.do_handshake()

    # Verify server info
    assert "serverInfo" in init_response
    server_info = init_response["serverInfo"]
    assert server_info["name"] == constants.SERVER_NAME
    assert "version" in server_info

    # Verify capabilities
    assert "capabilities" in init_response
    capabilities = init_response["capabilities"]
    assert "resources" in capabilities
    assert "prompts" in capabilities
    assert "tools" in capabilities
    assert "logging" in capabilities


@pytest.mark.asyncio
async def test_mcp_streaming(configured_client: MCPInProcSession) -> None:
    """Test MCP streaming operations."""
    # Call tool with progress tracking
    args = {"text": "test", "repeat": 2}  # Will repeat 'test' twice
    result = await configured_client.call_tool("example", args, with_progress=True)
    assert "content" in result
    assert len(result["content"]) == 1
    assert result["content"][0]["text"] == "testtest"


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# """Zed mode functionality for MCP server."""

# from __future__ import annotations

# from typing import TYPE_CHECKING, Any

# from llmling.prompts.models import (
#     BasePrompt,
#     DynamicPrompt,
#     FilePrompt,
#     PromptMessage,
#     PromptParameter,
#     StaticPrompt,
# )


# if TYPE_CHECKING:
#     from collections.abc import Sequence


# class ZedPromptMixin:
#     """Mixin providing Zed compatibility for prompts."""

#     original_prompt: BasePrompt

#     @property
#     def arguments(self) -> Sequence[PromptParameter]:
#         """Get simplified argument list for Zed."""
#         args = ", ".join(a.name for a in self.original_prompt.arguments)
#         # Only show single input argument
#         return [
#             PromptParameter(
#                 name="input",
#                 description=(
#                     "Format: 'first_arg :: key1=value1 | key2=value2' "
#                     f"(Original args: {args})"
#                 ),
#                 required=True,
#             )
#         ]


# class ZedStaticPrompt(ZedPromptMixin, StaticPrompt):
#     """Static prompt wrapper for Zed."""

#     async def format(
#         self, arguments: dict[str, Any] | None = None
#     ) -> list[PromptMessage]:
#         """Format messages using encoded arguments."""
#         decoded = decode_zed_args(arguments)
#         return await self.original_prompt.format(decoded)


# class ZedDynamicPrompt(ZedPromptMixin, DynamicPrompt):
#     """Dynamic prompt wrapper for Zed."""

#     async def format(
#         self, arguments: dict[str, Any] | None = None
#     ) -> list[PromptMessage]:
#         """Execute callable with decoded arguments."""
#         decoded = decode_zed_args(arguments)
#         return await self.original_prompt.format(decoded)


# class ZedFilePrompt(ZedPromptMixin, FilePrompt):
#     """File prompt wrapper for Zed."""

#     async def format(
#         self, arguments: dict[str, Any] | None = None
#     ) -> list[PromptMessage]:
#         """Format file content with decoded arguments."""
#         decoded = decode_zed_args(arguments)
#         return await self.original_prompt.format(decoded)


# def decode_zed_args(arguments: dict[str, Any] | None) -> dict[str, Any]:
#     """Decode Zed-encoded arguments.

#     Args:
#         arguments: Dictionary with encoded "input" parameter

#     Returns:
#         Decoded argument dictionary
#     """
#     if not arguments or "input" not in arguments:
#         return {}

#     input_str = arguments["input"]
#     if " :: " not in input_str:
#         # Single argument case
#         first_arg_name = next(iter(arguments)).name
#         return {first_arg_name: input_str}

#     # Multiple arguments case
#     parts = input_str.split(" :: ", 1)
#     first_arg_name = next((arg.name for arg in arguments if arg.required), "input")
#     result = {first_arg_name: parts[0]}

#     if len(parts) > 1:
#         for pair in parts[1].split(" | "):
#             if not pair:
#                 continue
#             key, value = pair.split("=", 1)
#             # Convert value types
#             match value.lower():
#                 case "true":
#                     result[key] = True
#                 case "false":
#                     result[key] = False
#                 case "null":
#                     result[key] = None
#                 case _:
#                     try:
#                         if "." in value:
#                             result[key] = float(value)
#                         else:
#                             result[key] = int(value)
#                     except ValueError:
#                         result[key] = value

#     return result


# def wrap_for_zed(prompt: BasePrompt) -> BasePrompt:
#     """Create Zed-compatible wrapper for prompt if needed.

#     Args:
#         prompt: Original prompt

#     Returns:
#         Wrapped prompt if multiple arguments, original otherwise
#     """
#     if len(prompt.arguments) <= 1:
#         return prompt

#     # Create appropriate wrapper based on prompt type
#     match prompt:
#         case StaticPrompt():
#             wrapper = ZedStaticPrompt(
#                 name=prompt.name,
#                 description=prompt.description,
#                 messages=prompt.messages,
#                 arguments=prompt.arguments,
#                 metadata=prompt.metadata,
#             )
#         case DynamicPrompt():
#             wrapper = ZedDynamicPrompt(
#                 name=prompt.name,
#                 description=prompt.description,
#                 import_path=prompt.import_path,
#                 template=prompt.template,
#                 completions=prompt.completions,
#                 arguments=prompt.arguments,
#                 metadata=prompt.metadata,
#             )
#         case FilePrompt():
#             wrapper = ZedFilePrompt(
#                 name=prompt.name,
#                 description=prompt.description,
#                 path=prompt.path,
#                 format=prompt.fmt,
#                 watch=prompt.watch,
#                 arguments=prompt.arguments,
#                 metadata=prompt.metadata,
#             )
#         case _:
#             msg = f"Unsupported prompt type: {type(prompt)}"
#             raise ValueError(msg)

#     wrapper.original_prompt = prompt
#     return wrapper



from __future__ import annotations

import asyncio

import httpx


YAML_CONFIG = """
resources:
  dynamic_resource:
    type: text
    content: "This was dynamically added!"

tools:
  example_tool:
    import_path: llmling.testing.tools.example_tool
    description: "A test tool"
"""


async def main() -> None:
    async with httpx.AsyncClient() as client:
        # First check what we have
        print("\nGetting initial components...")
        response = await client.get("http://localhost:8765/components")
        print(f"Current components: {response.json()}")

        # Inject new config
        print("\nInjecting new config...")
        yaml_config = """
        resources:
          dynamic_resource:
            type: text
            content: "This was dynamically added!"

        tools:
          example_tool:
            import_path: llmling.testing.tools.example_tool
            description: "A test tool"
        """

        import yaml

        config = yaml.safe_load(yaml_config)
        print(f"Sending config: {config}")

        response = await client.post("http://localhost:8765/inject-config", json=config)
        print(f"Response status: {response.status_code}")
        print(f"Response headers: {response.headers}")
        print(f"Raw response: {response.text}")

        try:
            print(f"JSON response: {response.json()}")
        except Exception as e:  # noqa: BLE001
            print(f"Failed to parse response as JSON: {e}")
        # Check components again
        print("\nChecking updated components...")
        response = await client.get("http://localhost:8765/components")
        print(f"Updated components: {response.json()}")

        # Try to load the new resource
        print("\nTrying to get our new resource...")
        response = await client.get("http://localhost:8765/resources")
        resources = response.json()
        if "dynamic_resource" in resources:
            print(f"Found our resource: {resources['dynamic_resource']}")

        # Check our new tool
        print("\nTrying to get our new tool...")
        response = await client.get("http://localhost:8765/tools")
        if response.status_code != 200:  # noqa: PLR2004
            print(f"Error getting tools: {response.text}")
        else:
            try:
                tools = response.json()
                if "example_tool" in tools:
                    print(f"Found our tool: {tools['example_tool']}")
                else:
                    print("Tool was not found in the response")
            except Exception as e:  # noqa: BLE001
                print(f"Error parsing tool response: {e}")
                print(f"Raw response: {response.text}")


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations

import asyncio

from llmling import Config, RuntimeConfig

from mcp_server_llmling import LLMLingServer


async def main() -> None:
    # Create minimal config
    res = {"type": "text", "content": "Initial resource"}
    config = Config.model_validate({"global_settings": {}, "resources": {"initial": res}})

    async with RuntimeConfig.from_config(config) as runtime:
        server = LLMLingServer(
            runtime,
            transport="stdio",
            enable_injection=True,  # Enable our injection server
            injection_port=8765,
        )
        print("Starting server with injection endpoint at http://localhost:8765")
        await server.start(raise_exceptions=True)


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations

from collections.abc import Sequence  # noqa: TC003
from typing import TYPE_CHECKING, Any, Literal

from fastapi import FastAPI, HTTPException, WebSocket, WebSocketDisconnect
from llmling.config.models import (
    CallableResource,
    CLIResource,
    ImageResource,
    PathResource,
    Resource,
    SourceResource,
    TextResource,
    ToolConfig,
)
from py2openai import OpenAIFunctionTool  # noqa: TC002
from pydantic import BaseModel
from pydantic.fields import Field

from mcp_server_llmling.log import get_logger
from mcp_server_llmling.transports.stdio import StdioServer


if TYPE_CHECKING:
    from mcp_server_llmling.server import LLMLingServer

# from mcp_server_llmling.ui import create_ui_app


logger = get_logger(__name__)


ComponentType = Literal["resource", "tool", "prompt"]


def create_app() -> FastAPI:
    """Create FastAPI application for config injection."""
    return FastAPI(
        title="LLMling Config Injection API",
        description="""
        API for hot-injecting configuration into running LLMling server.

        ## Features
        * Inject new resources
        * Update existing tools
        * Real-time configuration updates
        * WebSocket support for live updates

        ## WebSocket Interface
        Connect to `/ws` for real-time updates. The WebSocket interface supports:

        ### Message Types
        * update: Update components in real-time
        * query: Query current component status
        * error: Error reporting from client

        ### Message Format
        ```json
        {
            "type": "update|query|error",
            "data": {
                "resources": {...},
                "tools": {...}
            },
            "request_id": "optional-correlation-id"
        }
        ```

        ### Response Format
        ```json
        {
            "type": "success|error|update",
            "data": {...},
            "request_id": "correlation-id",
            "message": "optional status message"
        }
        ```
        """,
        version="1.0.0",
        openapi_tags=[
            {
                "name": "components",
                "description": "Server component operations (resources/tools/prompts)",
            },
            {"name": "config", "description": "Configuration management endpoints"},
        ],
        docs_url="/docs",
        redoc_url="/redoc",
        openapi_url="/openapi.json",
    )


class ComponentResponse(BaseModel):
    """Response model for component operations."""

    status: Literal["success", "error"]
    message: str
    component_type: ComponentType
    name: str


class ConfigUpdate(BaseModel):
    """Model for config updates."""

    resources: dict[str, Resource] | None = Field(
        default=None, description="Resource updates"
    )
    tools: dict[str, ToolConfig] | None = Field(default=None, description="Tool updates")


class BulkUpdateResponse(BaseModel):
    """Response model for bulk updates."""

    results: list[ComponentResponse]
    summary: dict[str, int] = Field(default_factory=lambda: {"success": 0, "error": 0})


class ConfigUpdateRequest(BaseModel):
    """Request model for config updates."""

    resources: dict[str, Resource] | None = None
    tools: dict[str, ToolConfig] | None = None
    replace_existing: bool = Field(
        default=True, description="Whether to replace existing components"
    )


class WebSocketMessage(BaseModel):
    """Message format for WebSocket communication."""

    type: Literal["update", "query", "error"]
    data: ConfigUpdateRequest | dict[str, Any]
    request_id: str | None = None


class WebSocketResponse(BaseModel):
    """Response format for WebSocket communication."""

    type: Literal["success", "error", "update"]
    data: ComponentResponse | list[ComponentResponse] | dict[str, Any]
    request_id: str | None = None
    message: str | None = None


class ConfigInjectionServer:
    """FastAPI server for hot config injection."""

    def __init__(
        self,
        llm_server: LLMLingServer,
        host: str = "localhost",
        port: int = 8765,
    ) -> None:
        """Initialize server.

        Args:
            llm_server: The LLMling server instance
            host: Host to bind to
            port: Port to listen on
        """
        self.llm_server = llm_server
        self.host = host
        self.port = port
        self.app = create_app()
        # create_ui_app(self.app)
        self._setup_routes()
        self._server: Any = None  # uvicorn server instance

    def _setup_routes(self) -> None:
        """Set up API routes."""

        @self.app.post(
            "/inject-config",
            response_model=ComponentResponse,
            tags=["config"],
            summary="Inject new configuration",
            description="Inject new configuration into the running server.",
            responses={
                200: {
                    "description": "Configuration successfully injected",
                    "content": {
                        "application/json": {
                            "example": {
                                "status": "success",
                                "message": "Config injected successfully",
                                "component_type": "resource",
                                "name": "example_resource",
                            }
                        }
                    },
                },
                400: {"description": "Invalid configuration"},
            },
        )
        async def inject_config(config: dict[str, Any]) -> ComponentResponse:
            """Inject raw YAML configuration."""
            logger.debug("Received config: %s", config)
            try:
                # Update resources
                if resources := config.get("resources"):
                    logger.debug("Processing resources: %s", resources)
                    for name, resource in resources.items():
                        # Validate based on resource type
                        resource_type = resource.get("type")
                        logger.debug(
                            "Processing resource %s of type %s", name, resource_type
                        )
                        match resource_type:
                            case "path":
                                validated = PathResource.model_validate(resource)
                            case "text":
                                validated = TextResource.model_validate(resource)
                            case "cli":
                                validated = CLIResource.model_validate(resource)
                            case "source":
                                validated = SourceResource.model_validate(resource)
                            case "callable":
                                validated = CallableResource.model_validate(resource)
                            case "image":
                                validated = ImageResource.model_validate(resource)
                            case _:
                                msg = f"Unknown resource type: {resource_type}"
                                raise ValueError(msg)  # noqa: TRY301

                        self.llm_server.runtime.register_resource(
                            name, validated, replace=True
                        )
                        logger.debug("Resource %s registered", name)

                # Update tools
                if tools := config.get("tools"):
                    logger.debug("Processing tools: %s", tools)
                    for name, tool in tools.items():
                        logger.debug("Processing tool: %s", name)
                        validated = ToolConfig.model_validate(tool)
                        self.llm_server.runtime._tool_registry.register(
                            name, validated, replace=True
                        )
                        logger.debug("Tool %s registered", name)

                result = ComponentResponse(
                    status="success",
                    message="Config injected successfully",
                    component_type="tool",
                    name="yaml_injection",
                )
                logger.debug("Returning response: %s", result.model_dump())
            except Exception as e:
                logger.exception("Failed to inject config")
                raise HTTPException(status_code=400, detail=str(e)) from e
            else:
                return result

        @self.app.get(
            "/components",
            tags=["components"],
            summary="List all components",
            description="Get a list of all registered components grouped by type.",
            response_description="Dictionary containing arrays of component names",
            responses={
                200: {
                    "description": "List of all components",
                    "content": {
                        "application/json": {
                            "example": {
                                "resources": ["resource1", "resource2"],
                                "tools": ["tool1", "tool2"],
                                "prompts": ["prompt1", "prompt2"],
                            }
                        }
                    },
                }
            },
        )
        async def list_components() -> dict[str, Sequence[str]]:
            """List all registered components."""
            return {
                "resources": self.llm_server.runtime.list_resource_names(),
                "tools": self.llm_server.runtime.list_tool_names(),
                "prompts": self.llm_server.runtime.list_prompt_names(),
            }

        # Resource endpoints
        @self.app.post(
            "/resources/{name}",
            response_model=ComponentResponse,
            tags=["components"],
            summary="Add or update resource",
            description="""
            Register a new resource or update an existing one.
            Supports various resource types including path, text, CLI, source,
            callable, and image.
            """,
            responses={
                200: {"description": "Resource successfully registered"},
                400: {"description": "Invalid resource configuration"},
            },
        )
        async def add_resource(name: str, resource: Resource) -> ComponentResponse:
            """Add or update a resource."""
            try:
                self.llm_server.runtime.register_resource(name, resource, replace=True)
                return ComponentResponse(
                    status="success",
                    message=f"Resource {name} registered",
                    component_type="resource",
                    name=name,
                )
            except Exception as e:
                raise HTTPException(status_code=400, detail=str(e)) from e

        @self.app.get(
            "/resources",
            tags=["components"],
            summary="List all resources",
            description="Get a list of all registered resources with their full config.",
            responses={
                200: {
                    "description": "Dictionary of resources",
                    "content": {
                        "application/json": {
                            "example": {
                                "resource1": {
                                    "type": "text",
                                    "content": "Example content",
                                },
                                "resource2": {"type": "path", "path": "/example/path"},
                            }
                        }
                    },
                }
            },
        )
        async def list_resources() -> dict[str, Resource]:
            """List all resources with their configuration."""
            return {
                name: self.llm_server.runtime._resource_registry[name]
                for name in self.llm_server.runtime.list_resource_names()
            }

        @self.app.delete(
            "/resources/{name}",
            response_model=ComponentResponse,
            tags=["components"],
            summary="Remove resource",
            description="Remove a registered resource by name.",
            responses={
                200: {"description": "Resource successfully removed"},
                404: {"description": "Resource not found"},
            },
        )
        async def remove_resource(name: str) -> ComponentResponse:
            """Remove a resource."""
            try:
                del self.llm_server.runtime._resource_registry[name]
                return ComponentResponse(
                    status="success",
                    message=f"Resource {name} removed",
                    component_type="resource",
                    name=name,
                )
            except KeyError as e:
                raise HTTPException(
                    status_code=404, detail=f"Resource {name} not found"
                ) from e

        # Tool endpoints
        @self.app.post(
            "/tools/{name}",
            response_model=ComponentResponse,
            tags=["components"],
            summary="Add or update tool",
            description="Register a new tool or update an existing one.",
            responses={
                200: {"description": "Tool successfully registered"},
                400: {"description": "Invalid tool configuration"},
            },
        )
        async def add_tool(name: str, tool: ToolConfig) -> ComponentResponse:
            """Add or update a tool."""
            try:
                self.llm_server.runtime._tool_registry.register(name, tool, replace=True)
                return ComponentResponse(
                    status="success",
                    message=f"Tool {name} registered",
                    component_type="tool",
                    name=name,
                )
            except Exception as e:
                raise HTTPException(status_code=400, detail=str(e)) from e

        @self.app.get(
            "/tools",
            tags=["components"],
            summary="List all tools",
            description="Get a list of all registered tools with their OpenAPI schemas.",
            responses={
                200: {
                    "description": "Dictionary of tools with their schemas",
                    "content": {
                        "application/json": {
                            "example": {
                                "tool1": {
                                    "name": "tool1",
                                    "description": "Example tool",
                                    "parameters": {
                                        "type": "object",
                                        "properties": {},
                                    },
                                }
                            }
                        }
                    },
                },
                500: {"description": "Failed to get tool schemas"},
            },
        )
        async def list_tools() -> dict[str, OpenAIFunctionTool]:
            """List all tools with their OpenAPI schemas."""
            try:
                return {
                    name: tool.get_schema()
                    for name, tool in self.llm_server.runtime.tools.items()
                }
            except Exception as e:
                raise HTTPException(
                    status_code=500, detail=f"Failed to get tool schemas: {e}"
                ) from e

        @self.app.delete(
            "/tools/{name}",
            response_model=ComponentResponse,
            tags=["components"],
            summary="Remove tool",
            description="Remove a registered tool by name.",
            responses={
                200: {"description": "Tool successfully removed"},
                404: {"description": "Tool not found"},
            },
        )
        async def remove_tool(name: str) -> ComponentResponse:
            """Remove a tool."""
            try:
                del self.llm_server.runtime._tool_registry[name]
                return ComponentResponse(
                    status="success",
                    message=f"Tool {name} removed",
                    component_type="tool",
                    name=name,
                )
            except KeyError as e:
                raise HTTPException(
                    status_code=404, detail=f"Tool {name} not found"
                ) from e

        # Bulk update endpoint
        @self.app.post(
            "/bulk-update",
            response_model=BulkUpdateResponse,
            tags=["config"],
            summary="Bulk update components",
            description="""
            Update multiple components in a single request.

            This endpoint allows you to register multiple resources and tools at once.
            Failed operations will be reported in the response but won't affect others.
            """,
            responses={
                200: {
                    "description": "Bulk update results",
                    "content": {
                        "application/json": {
                            "example": {
                                "results": [
                                    {
                                        "status": "success",
                                        "message": "Resource registered",
                                        "component_type": "resource",
                                        "name": "example",
                                    }
                                ],
                                "summary": {"success": 1, "error": 0},
                            }
                        }
                    },
                },
            },
        )
        async def bulk_update(request: ConfigUpdateRequest) -> BulkUpdateResponse:
            """Update multiple components at once."""
            responses: list[ComponentResponse] = []
            summary = {"success": 0, "error": 0}

            if request.resources:
                for name, resource in request.resources.items():
                    try:
                        self.llm_server.runtime.register_resource(
                            name, resource, replace=request.replace_existing
                        )
                        response = ComponentResponse(
                            status="success",
                            message=f"Resource {name} registered",
                            component_type="resource",
                            name=name,
                        )
                        responses.append(response)
                        summary["success"] += 1
                    except Exception as e:  # noqa: BLE001
                        response = ComponentResponse(
                            status="error",
                            message=str(e),
                            component_type="resource",
                            name=name,
                        )
                        responses.append(response)
                        summary["error"] += 1

            if request.tools:
                for name, tool in request.tools.items():
                    try:
                        self.llm_server.runtime._tool_registry.register(
                            name, tool, replace=request.replace_existing
                        )
                        response = ComponentResponse(
                            status="success",
                            message=f"Tool {name} registered",
                            component_type="tool",
                            name=name,
                        )
                        responses.append(response)
                        summary["success"] += 1
                    except Exception as e:  # noqa: BLE001
                        response = ComponentResponse(
                            status="error",
                            message=str(e),
                            component_type="tool",
                            name=name,
                        )
                        responses.append(response)
                        summary["error"] += 1

            return BulkUpdateResponse(results=responses, summary=summary)

        # WebSocket endpoint
        @self.app.websocket(
            "/ws",
            name="component_updates",
            dependencies=None,
        )
        async def websocket_endpoint(websocket: WebSocket) -> None:
            """Handle WebSocket connections."""
            await websocket.accept()
            try:
                while True:
                    raw_data = await websocket.receive_json()
                    try:
                        message = WebSocketMessage.model_validate(raw_data)
                        match message.type:
                            case "update":
                                if isinstance(message.data, dict):
                                    request = ConfigUpdateRequest.model_validate(
                                        message.data
                                    )
                                    response = await bulk_update(request)
                                    await websocket.send_json(
                                        WebSocketResponse(
                                            type="success",
                                            data=response.results,
                                            request_id=message.request_id,
                                            message="Components updated successfully",
                                        ).model_dump()
                                    )
                            case "query":
                                # Handle component queries
                                components = await list_components()
                                await websocket.send_json(
                                    WebSocketResponse(
                                        type="success",
                                        data=components,
                                        request_id=message.request_id,
                                    ).model_dump()
                                )
                            case "error":
                                logger.error("Client error: %s", message.data)
                    except Exception:
                        error_msg = "Operation failed"
                        logger.exception(error_msg)
                        await websocket.send_json(
                            WebSocketResponse(
                                type="error",
                                data={},
                                message=error_msg,
                                request_id=getattr(message, "request_id", None),
                            ).model_dump()
                        )
            except WebSocketDisconnect:
                logger.debug("WebSocket client disconnected")

    async def start(self) -> None:
        """Start FastAPI server in the same event loop."""
        if not isinstance(self.llm_server.transport, StdioServer):
            msg = "Config injection requires stdio transport"
            raise RuntimeError(msg)  # noqa: TRY004

        import uvicorn

        config = uvicorn.Config(
            self.app,
            host=self.host,
            port=self.port,
            log_level="info",
        )
        self._server = uvicorn.Server(config)
        # Run in same event loop
        await self._server.serve()

    async def stop(self) -> None:
        """Stop FastAPI server."""
        if self._server:
            self._server.should_exit = True
            await self._server.shutdown()
            self._server = None


if __name__ == "__main__":

    async def main() -> None:
        import httpx

        async with httpx.AsyncClient() as client:
            # Add a single resource
            response = await client.post(
                "http://localhost:8765/resources/my_resource",
                json={"type": "text", "content": "Dynamic content"},
            )
            print(response.json())

            # Add a tool
            url = "http://localhost:8765/tools/my_tool"
            response = await client.post(url, json={"import_path": "myapp.tools.analyze"})
            print(response.json())

            # List all components
            components = await client.get("http://localhost:8765/components")
            print(components.json())

            # Bulk update
            response = await client.post(
                "http://localhost:8765/bulk-update",
                json={
                    "resources": {
                        "resource1": {"type": "text", "content": "Content 1"},
                        "resource2": {"type": "text", "content": "Content 2"},
                    },
                    "tools": {
                        "tool1": {"import_path": "myapp.tools.tool1"},
                        "tool2": {"import_path": "myapp.tools.tool2"},
                    },
                },
            )
            print(response.json())


if __name__ == "__main__":
    import asyncio

    from llmling import Config, RuntimeConfig

    from mcp_server_llmling.server import LLMLingServer

    async def main() -> None:
        # Create minimal config
        config = Config.model_validate({
            "global_settings": {},
            "resources": {"initial": {"type": "text", "content": "Initial resource"}},
        })

        async with RuntimeConfig.from_config(config) as runtime:
            server = LLMLingServer(
                runtime,
                transport="stdio",
                enable_injection=True,  # Enable our injection server
                injection_port=8765,
            )
            print("Starting server with injection endpoint at http://localhost:8765")
            await server.start(raise_exceptions=True)

    asyncio.run(main())



"""MCP server CLI interface."""

from __future__ import annotations

import asyncio
from enum import Enum
import logging
import os
from pathlib import Path
import sys
from typing import TYPE_CHECKING, Any

from llmling import RuntimeConfig, config_resources
import typer as t

from mcp_server_llmling import __version__, constants
from mcp_server_llmling.log import get_logger
from mcp_server_llmling.server import LLMLingServer


if TYPE_CHECKING:
    from pathlib import Path

logger = get_logger(__name__)

cli = t.Typer(
    name="MCP Server",
    help=(
        "🚀 MCP Server CLI interface. Run and manage the MCP protocol server! 🚀\n\n"
        "Check out https://github.com/your-repo/mcp-server-llmling !"
    ),
    no_args_is_help=True,
)

# Command groups for list subcommand
list_cli = t.Typer(help="List available components.", no_args_is_help=True)
cli.add_typer(list_cli, name="list")

# Common option definitions
CONFIG_HELP = "Path to LLMling configuration file"
TRANSPORT_HELP = "Transport type (stdio or sse)"
HOST_HELP = "Host address for SSE transport"
PORT_HELP = "Port number for SSE transport"
INJ_PORT_HELP = "Port for config injection server"
INJ_HOST_HELP = "Host for config injection server"
NAME_HELP = "Server name for MCP protocol"
TIMEOUT_HELP = "Connection timeout in seconds"
VERBOSE_HELP = "Enable verbose output"
QUIET_HELP = "Suppress non-essential output"

# Option command tuples
CONFIG_CMDS = "-c", "--config"
TRANSPORT_CMDS = "-t", "--transport"
HOST_CMDS = "-h", "--host"
PORT_CMDS = "-p", "--port"
NAME_CMDS = "-n", "--name"
VERBOSE_CMDS = "-v", "--verbose"
QUIET_CMDS = "-q", "--quiet"


class LogLevel(str, Enum):
    """Valid logging levels."""

    DEBUG = "debug"
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"


def validate_transport(value: str) -> str:
    """Validate transport type."""
    if value not in {"stdio", "sse"}:
        msg = f"Invalid transport type: {value}. Must be 'stdio' or 'sse'"
        raise t.BadParameter(msg)
    return value


def version_callback(value: bool) -> None:
    """Print version and exit if --version is used."""
    if value:
        t.echo(f"MCP Server version: {__version__}")
        raise t.Exit


def verbose_callback(ctx: t.Context, _param: t.CallbackParam, value: bool) -> bool:
    """Set up verbose logging."""
    if value:
        logging.getLogger().setLevel(logging.DEBUG)
    return value


def quiet_callback(ctx: t.Context, _param: t.CallbackParam, value: bool) -> bool:
    """Set up quiet logging."""
    if value:
        logging.getLogger().setLevel(logging.ERROR)
    return value


@cli.command()
def start(
    config_path: Path = t.Argument(  # noqa: B008
        config_resources.TEST_CONFIG,  # ""..."" for required
        help=CONFIG_HELP,
        exists=True,
    ),
    transport: str = t.Option(
        "stdio",
        *TRANSPORT_CMDS,
        help=TRANSPORT_HELP,
        callback=validate_transport,
    ),
    host: str = t.Option("localhost", *HOST_CMDS, help=HOST_HELP),
    port: int = t.Option(8000, *PORT_CMDS, help=PORT_HELP),
    injection_host: str = t.Option("localhost", help=INJ_HOST_HELP),
    injection_port: int = t.Option(8765, help=INJ_PORT_HELP),
    server_name: str = t.Option(constants.SERVER_NAME, *NAME_CMDS, help=NAME_HELP),
    log_level: LogLevel = t.Option(LogLevel.INFO, "-l", "--log-level"),  # noqa: B008
    timeout: float = t.Option(30.0, help=TIMEOUT_HELP),
    enable_injection: bool = t.Option(False, help="Enable config injection server"),
    verbose: bool = t.Option(
        False, *VERBOSE_CMDS, help=VERBOSE_HELP, callback=verbose_callback
    ),
    quiet: bool = t.Option(False, *QUIET_CMDS, help=QUIET_HELP, callback=quiet_callback),
    version: bool = t.Option(None, "--version", callback=version_callback, is_eager=True),  # type: ignore
) -> None:
    """Start the MCP protocol server."""
    try:
        logging.getLogger().setLevel(log_level.value.upper())
        transport_options: dict[str, Any] = {}
        if transport == "sse":
            transport_options = {"host": host, "port": port}

        with RuntimeConfig.open_sync(config_path) as runtime:
            logger.debug("Created runtime with config: %s", runtime._config)
            server = LLMLingServer(
                runtime=runtime,
                transport=transport,  # type: ignore
                name=server_name,
                transport_options=transport_options,
                enable_injection=enable_injection,
                injection_port=injection_port,
            )
            asyncio.run(server.start(raise_exceptions=True))
    except KeyboardInterrupt:
        logger.info("Server stopped by user")
    except Exception:
        logger.exception("Server startup failed")
        sys.exit(1)


@cli.command()
def info(
    verbose: bool = t.Option(
        False, *VERBOSE_CMDS, help=VERBOSE_HELP, callback=verbose_callback
    ),
):
    """Show server information and capabilities."""
    info_data = {
        "Version": __version__,
        "Python": sys.version,
        "Platform": sys.platform,
        "Supported transports": ["stdio", "sse"],
        "Default ports": {
            "SSE": 8000,
            "Injection": 8765,
        },
    }
    for key, value in info_data.items():
        t.echo(f"{key}: {value}")


@cli.command()
def inject(
    config_path: Path = t.Argument(  # noqa: B008
        ..., help="Path to configuration file to inject", exists=True
    ),
    host: str = t.Option("localhost", *HOST_CMDS, help=HOST_HELP),
    port: int = t.Option(8765, *PORT_CMDS, help=PORT_HELP),
    verbose: bool = t.Option(
        False, *VERBOSE_CMDS, help=VERBOSE_HELP, callback=verbose_callback
    ),
):
    """Inject configuration into a running server."""
    import httpx

    try:
        with config_path.open("rb") as f:
            data = f.read()
        url = f"http://{host}:{port}/inject-config"
        response = httpx.post(url, content=data)
        response.raise_for_status()
        t.echo("Configuration injected successfully")
    except Exception as e:  # noqa: BLE001
        t.echo(f"Failed to inject configuration: {e}", err=True)
        sys.exit(1)


def run() -> None:
    """Entry point for the CLI."""
    if os.name == "nt":  # Windows
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
    cli()


if __name__ == "__main__":
    run()



"""Test server lifecycle using different approaches."""

from __future__ import annotations

import asyncio
import contextlib
import json
from typing import TYPE_CHECKING, Any

import logfire
import pytest

from mcp_server_llmling import constants


if TYPE_CHECKING:
    from llmling.config.runtime import RuntimeConfig

    from mcp_server_llmling import LLMLingServer
    from mcp_server_llmling.mcp_inproc_session import MCPInProcSession


# Initialize logfire to avoid warnings
logfire.configure()


@pytest.mark.asyncio
async def test_server_lifecycle_handshake_client(client: MCPInProcSession) -> None:
    """Test server lifecycle using MCPInProcSession."""
    try:
        await client.start()
        await asyncio.sleep(0.5)

        # Initialize connection
        init_response = await client.send_request(
            "initialize",
            {
                "protocolVersion": "2024-11-05",
                "capabilities": {},
                "clientInfo": {"name": "test-client", "version": "1.0"},
            },
        )
        assert isinstance(init_response, dict)
        assert "serverInfo" in init_response
        server_info = init_response["serverInfo"]
        assert server_info["name"] == constants.SERVER_NAME
        assert "version" in server_info
        assert "capabilities" in init_response

        # Send initialized notification
        await client.send_notification("notifications/initialized", {})

        # Test functionality
        tools = await client.list_tools()
        assert isinstance(tools, list)
        resources = await client.list_resources()
        assert isinstance(resources, list)
        prompts = await client.list_prompts()
        assert isinstance(prompts, list)
    finally:
        await client.close()


@pytest.mark.asyncio
async def test_server_lifecycle_test_session(
    running_server: tuple[LLMLingServer, tuple[Any, Any]],
) -> None:
    """Test server lifecycle using test session."""
    server, (client_read, client_write) = running_server

    # Create proper MCP message
    from mcp.types import JSONRPCMessage, JSONRPCRequest

    init_request = JSONRPCMessage(
        JSONRPCRequest(
            jsonrpc="2.0",
            id=1,
            method="initialize",
            params={
                "protocolVersion": "2024-11-05",
                "capabilities": {},
                "clientInfo": {"name": "test-client", "version": "1.0"},
            },
        )
    )

    await client_write.send(init_request)
    response = await client_read.receive()
    assert "result" in response.root.model_dump()
    result = response.root.result
    assert "serverInfo" in result
    assert result["serverInfo"]["name"] == constants.SERVER_NAME

    # Send initialized notification
    from mcp.types import JSONRPCNotification

    notification = JSONRPCMessage(
        JSONRPCNotification(
            jsonrpc="2.0",
            method="notifications/initialized",
            params={},
        )
    )
    await client_write.send(notification)

    # Test functionality
    tools_request = JSONRPCMessage(
        JSONRPCRequest(
            jsonrpc="2.0",
            id=2,
            method="tools/list",
            params={},
        )
    )
    await client_write.send(tools_request)
    tools_response = await client_read.receive()
    assert "tools" in tools_response.root.result


@pytest.mark.asyncio
async def test_server_lifecycle_direct(
    runtime_config: RuntimeConfig,
    server: LLMLingServer,
) -> None:
    """Test server lifecycle using direct method calls."""
    try:
        # Start runtime components
        await runtime_config.startup()

        # Test direct access
        tools = runtime_config.get_tools()
        assert isinstance(tools, list)
        assert len(tools) > 0  # Should have our test tools

        resources = runtime_config.list_resource_names()
        assert isinstance(resources, list)

        prompts = runtime_config.list_prompt_names()
        assert isinstance(prompts, list)

    finally:
        await runtime_config.shutdown()
        await server.shutdown()


@pytest.mark.asyncio
async def test_server_lifecycle_subprocess() -> None:
    """Test server lifecycle using raw subprocess."""
    process = await asyncio.create_subprocess_exec(
        *constants.SERVER_CMD,
        stdin=asyncio.subprocess.PIPE,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
    )

    # Start stderr reader task
    async def read_stderr() -> None:
        assert process.stderr
        while True:
            line = await process.stderr.readline()
            if not line:
                break
            print(f"Server stderr: {line.decode().strip()}")

    stderr_task = asyncio.create_task(read_stderr())

    try:
        assert process.stdin
        assert process.stdout
        await asyncio.sleep(0.5)  # Give server time to start

        # Send initialize request
        request = {
            "jsonrpc": "2.0",
            "id": 1,
            "method": "initialize",
            "params": {
                "protocolVersion": "2024-11-05",
                "capabilities": {},
                "clientInfo": {"name": "test-client", "version": "1.0"},
            },
        }
        process.stdin.write(json.dumps(request).encode() + b"\n")
        await process.stdin.drain()

        # Read until we get a valid JSON response
        while True:
            response = await process.stdout.readline()
            if not response:
                msg = "No response from server"
                raise RuntimeError(msg)

            try:
                result = json.loads(response.decode())
                assert "result" in result
                assert "serverInfo" in result["result"]
                assert result["result"]["serverInfo"]["name"] == constants.SERVER_NAME
                break  # Valid response found
            except json.JSONDecodeError:
                continue  # Skip non-JSON lines

    finally:
        # Cleanup
        stderr_task.cancel()
        with contextlib.suppress(asyncio.CancelledError):
            await stderr_task
        process.terminate()
        try:
            await asyncio.wait_for(process.wait(), timeout=1.0)
        except TimeoutError:
            process.kill()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Function parameter encoding utils. Used for Zed to overcome one-argument limitation."""

from __future__ import annotations

from functools import wraps
import inspect
from typing import TYPE_CHECKING, Any, ParamSpec, TypeVar


if TYPE_CHECKING:
    from collections.abc import Callable


P = ParamSpec("P")
R = TypeVar("R")


def with_encoded_params(
    func: Callable[P, R],
) -> Callable[P, R] | Callable[[str], R]:
    """Wrap a function to accept encoded parameters as a single string.

    Only wraps functions with more than one parameter. Functions with 0 or 1
    parameters are returned as-is.

    Format: "arg :: key1=value1 | key2=value2"
    The part before :: is the first argument, after are key-value pairs.

    Args:
        func: The function to wrap.

    Returns:
        Either the original function (if 0-1 params) or a wrapped version.
    """
    sig = inspect.signature(func)
    param_count = len(sig.parameters)

    # Don't wrap functions with 0 or 1 parameters
    if param_count <= 1:
        return func

    @wraps(func)
    def wrapper(param_string: str) -> R:
        # Split arg and kwargs
        parts = param_string.split(" :: ", 1)
        first_arg = parts[0]

        if len(parts) == 1:
            return func(first_arg)  # type: ignore

        # Parse kwargs
        kwargs: dict[str, Any] = {}
        for pair in parts[1].split(" | "):
            if not pair:
                continue
            key, value = pair.split("=", 1)
            # Convert value to appropriate type
            match value.lower():
                case "true":
                    kwargs[key] = True
                case "false":
                    kwargs[key] = False
                case "null":
                    kwargs[key] = None
                case _:
                    try:
                        if "." in value:
                            kwargs[key] = float(value)
                        else:
                            kwargs[key] = int(value)
                    except ValueError:
                        kwargs[key] = value

        return func(first_arg, **kwargs)  # type: ignore

    # Create new signature and docstring for the wrapper
    orig_sig = inspect.signature(func)
    orig_doc = func.__doc__ or ""

    # Create a new docstring that includes both wrapper and original info
    wrapper.__doc__ = f"""Wrapped version of {func.__name__}.

    Args:
        param_string: Encoded parameters in format: "arg :: key=value | key2=value2"
                     where 'arg' is passed as the first argument and the rest
                     are passed as keyword arguments.

    Original function:
        {orig_doc}

    Original signature:
        {func.__name__}{orig_sig}
    """

    # Update the wrapper's signature
    wrapper.__signature__ = inspect.Signature(  # type: ignore
        [
            inspect.Parameter(
                "param_string",
                inspect.Parameter.POSITIONAL_OR_KEYWORD,
                annotation=str,
            )
        ]
    )

    return wrapper


if __name__ == "__main__":

    @with_encoded_params
    def process_item(
        item: str,
        color: str = "red",
        count: int = 1,
        active: bool = True,
    ) -> str:
        return f"Processing {count} {color} {item}(s), active: {active}"

    # Examples
    result = process_item("box :: color=blue | count=3")
    result = process_item("container :: active=false | count=5 | color=green")
    result = process_item("sphere")  # just arg, no kwargs
